@@ -261,7 +261,10 @@ with magnification @var{mag} of the string @var{text}."
              ((eq? my-code 'capo)
               (set! capo-fret (cadr my-item)))
              ((eq? my-code 'place-fret)
-              (set! dot-list (cons* (cdr my-item) dot-list))))
+              (set! dot-list (cons* (cdr my-item) dot-list)))
+             (else (ly:warning
+                     "unknown fret diagram placement code: ~a"
+                     (object->string my-code))))
             (parse-item (cdr mylist)))))
     ;; calculate fret-range
     (let ((maxfret 0)
@@ -1009,7 +1012,7 @@ a fret-indication list with the appropriate values"
     ;; add the modified details
     (set! props
           (prepend-alist-chain 'fret-diagram-details details props))
-    `(,props . ,output-list))) ;ugh -- hard-coded spell -- procedure better
+    (cons props output-list)))
 
 (define-public
   (fret-parse-terse-definition-string props definition-string)
@@ -1050,14 +1053,17 @@ a fret-indication list with the appropriate values"
                                 (list-head this-list max-element-index))))
                     (if (equal? last-element ")")
                         (let* ((this-barre
-                                (get-sub-list fret barre-start-list))
-                               (insert-index (- (length this-barre) 1)))
-                          (set! output-list
+                                (get-sub-list fret barre-start-list)))
+                          (if (not this-barre)
+                            (ly:warning
+                              "Ignoring barre end without beginning on fret: ~a"
+                              fret)
+                            (set! output-list
                                 (cons-fret (cons* 'barre
                                                   (car this-barre)
                                                   current-string
                                                   (cdr this-barre))
-                                           output-list))
+                                           output-list)))
                           (set! this-list
                                 (list-head this-list max-element-index))))
                     (if (number? fret)
@@ -1083,7 +1089,7 @@ a fret-indication list with the appropriate values"
             (parse-item (cdr myitems)))))
     (set! details (acons 'string-count string-count details))
     (set! props (prepend-alist-chain 'fret-diagram-details details props))
-    `(,props . ,output-list))) ; ugh -- hard coded; proc is better
+    (cons props output-list)))
 
 
 (define-markup-command
